{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ComCtrls,
  ExtCtrls, Spin, Menus, uUDPClient, uUDPServer,
  uSmartDesktopImage, TypInfo,
  uSmartTrayIcon, uClientInfo, uUDPComunicationClient, uUDPComunicationServer;

const
     COUNT_ROW_ZONE_SCREEN = 2;
     COUNT_COL_ZONE_SCREEN = 2;

type

  { TFrmMain }

  TFrmMain = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Chk_Remote_Control: TCheckBox;
    Chk_Jumbo: TCheckBox;
    Chk_With_Mouse_Position: TCheckBox;
    Chk_SendScreen: TCheckBox;
    Label7: TLabel;
    SpinEdit_No_Jumbo_Dimension: TSpinEdit;
    Timer_Resize: TTimer;
    Txt_LastKey_Values: TEdit;
    Txt_LastMouse_Values: TEdit;
    Img_OpenPopUp: TImage;
    Label4: TLabel;
    Memo2: TMemo;
    Panel_PopUp: TPanel;
    TabSheet3: TTabSheet;
    Timer_Click: TTimer;
    TBtn_CapsLock: TToggleBox;
    Tbtn_Shift: TToggleBox;
    Tbtn_Alt: TToggleBox;
    TBtn_Ctrl: TToggleBox;
    TBtn_AltGr: TToggleBox;
    TBtn_Num: TToggleBox;
    Txt_Pwd: TEdit;
    ImageList1: TImageList;
    Label2: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    MenuItem1: TMenuItem;
    PopupMenu1: TPopupMenu;
    Timer1: TTimer;
    TrackBar1: TTrackBar;
    TrackBar2: TTrackBar;
    Txt_Host: TEdit;
    Label1: TLabel;
    Label3: TLabel;
    Memo1: TMemo;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Txt_Port_Sending: TEdit;
    Txt_Remote_Port_Reception: TEdit;
    Image1: TImage;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormResize(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1Paint(Sender: TObject);
    procedure Image1Resize(Sender: TObject);
    procedure Img_OpenPopUpClick(Sender: TObject);
    procedure MenuItem1Click(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure TBtn_CapsLockChange(Sender: TObject);
    procedure TBtn_CtrlChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer_ClickTimer(Sender: TObject);
    procedure Timer_ResizeTimer(Sender: TObject);
  private
         MySmartTrayIcon         : TSmartTrayIcon;
         UDPClientThread1        : array[0..((COUNT_ROW_ZONE_SCREEN*COUNT_COL_ZONE_SCREEN)-1)] of TUDPClient;
         UDPServerThread1        : array[0..((COUNT_ROW_ZONE_SCREEN*COUNT_COL_ZONE_SCREEN)-1)] of TUDPServer;
         UDPComunicationClient   : TUDPComunicationClient;
         UDPComunicationServer   : TUDPComunicationServer;
         NumClick                : integer;
         MouseButton             : string;
         ScreenAdjusted          : boolean;
         procedure ClientNotifyError(Sender: TObject; id_error : integer; desc_error : string);
         procedure ServerNotifyError(Sender: TObject; id_error : integer; desc_error : string);
  public
         UDPCriticalSection       : TRTLCriticalSection;
         UDPCriticalSectionMouse  : TRTLCriticalSection;
         UDPClientMessage         : string;
         UDPClientIdx             : integer;
         UDPServerMessage         : string;
         UDPServerIdx             : integer;
         sImg                     : TSmartDesktopImage;
         RemoteScreenWidth        : integer;
         RemoteScreenHeight       : integer;
         MouseMsg_Button          : string;
         MouseMsg_Width           : integer;
         MouseMsg_Height          : integer;
         MouseMsg_X               : integer;
         MouseMsg_Y               : integer;
         MouseShift               : TShiftState;
         MemoryJpg                : TMemoryStream;
         LastKeyPressd            : string;
         LastShiftPressed         : string;
         StopUploadImage          : boolean;
         {$ifdef GetScreenInMainThread}
         Timer1_start : boolean;
         ByPassTimer  : boolean;
         MyScreen     : TJpegImage;
         MyScreenW    : integer;
         MyScreenH    : integer;
         ThreadZone   : integer;
         ZoneB64      : string;
         {$endif}

         procedure UDPServerShowImage;
         procedure UDPClientGetScreen;
         procedure UDPActionMouse;
         procedure UDPActionKey;
         procedure AdjustScreenImage;
  end;

var
  FrmMain: TFrmMain;

implementation
uses
    MouseAndKeyInput, LCLType;

{$R *.lfm}

{ TFrmMain }

procedure TFrmMain.FormCreate(Sender: TObject);
begin
     Self.KeyPreview       := true; //to get key press

     Img_OpenPopUp.visible := false;
     Panel_PopUp.Visible   := false;

     {$ifdef GetScreenInMainThread}
         MyScreen     := TJpegImage.Create;
     {$endif}

     MemoryJpg        := TMemoryStream.Create;

     MySmartTrayIcon := TSmartTrayIcon.Create(Self, PopupMenu1);
     NumClick        := 0;

     InitCriticalSection(UDPCriticalSection);
     InitCriticalSection(UDPCriticalSectionMouse);


     Self.Memo1.Append ( 'Internet IP: ' + GetIpPub() );
     Self.Memo1.Append ( '' );
     Self.Memo1.Append ( 'Host name: ' + myLocalComputerName() );
     Self.Memo1.Append ( '' );
     Self.Memo1.Append ( 'Local IP info: ' );
     Self.Memo1.Append ( myLocalIPAddress() );
     Self.Memo1.Append ( '' );
end;

procedure TFrmMain.Button1Click(Sender: TObject);
var
   i      : integer;
   port_r : integer;
   port_s : integer;
   sizep  : integer;
begin

     if Chk_Jumbo.Checked then
        sizep := 65507
     else
         sizep := SpinEdit_No_Jumbo_Dimension.Value;

     Self.Chk_SendScreen.Enabled               := false;
     Self.Chk_With_Mouse_Position.Enabled      := false;
     Self.TrackBar1.Enabled                    := false;
     Self.Txt_Port_Sending.Enabled             := false;
     Self.Txt_Remote_Port_Reception.Enabled    := false;
     Self.Txt_Host.Enabled                     := false;
     Chk_Jumbo.Enabled                         := false;
     Self.TrackBar2.Enabled                    := false;
     Self.Txt_Pwd.Enabled                      := false;

     {$ifdef GetScreenInMainThread}
             if Self.Chk_SendScreen.Checked then
             begin
                  Self.Timer1.Interval := 10;
                  ByPassTimer          := false;
             end;
     {$endif}

     port_r := StrToIntDef(Self.Txt_Remote_Port_Reception.Text, 5000);
     port_s := StrToIntDef(Self.Txt_Port_Sending.Text, 8000);
     for i:=0 to ((COUNT_ROW_ZONE_SCREEN*COUNT_COL_ZONE_SCREEN)-1) do
     begin

          if not Self.Chk_SendScreen.Checked then
          begin
               if UDPServerThread1[i]=nil then
               begin
                    ScreenAdjusted     := false;
                    StopUploadImage    := false;
                    Self.Image1.Top    := 1;
                    Self.Image1.Left   := 1;
                    Self.Image1.Width  := Self.TabSheet2.Width-2;
                    Self.Image1.Height := Self.TabSheet2.Height-2;

                    UDPServerThread1[i]:=TUDPServer.Create(IntToStr(port_s + i), Txt_Host.text, IntToStr(port_r + i), Self.Txt_Pwd.Text, COUNT_ROW_ZONE_SCREEN, COUNT_COL_ZONE_SCREEN, i);
                    UDPServerThread1[i].OnNotifyError:=@ServerNotifyError;
                    UDPServerThread1[i].SetTimeOut(50);
                    UDPServerThread1[i].Start;
               end;
          end else begin
               if UDPClientThread1[i]=nil then
               begin
                    UDPClientThread1[i]:=TUDPClient.Create(IntToStr(port_s + i), Self.Txt_Pwd.Text, true, Chk_With_Mouse_Position.Checked, Self.TrackBar1.Position , COUNT_ROW_ZONE_SCREEN, COUNT_COL_ZONE_SCREEN, i, sizep, Self.TrackBar2.Position);
                    UDPClientThread1[i].AddRemoteIpPort(Self.Txt_Host.Text, (port_r + i));
                    UDPClientThread1[i].OnNotifyError:=@ClientNotifyError;
                    UDPClientThread1[i].Start;
               end;
          end;

     end;

     Inc(i);

     if not Self.Chk_SendScreen.Checked then
     begin
          if UDPComunicationClient=nil then
          begin
               Self.MouseMsg_Button        := 'MV';
               UDPComunicationClient       := TUDPComunicationClient.Create(IntToStr(port_s + i),Txt_Host.text, IntToStr(port_r + i), Self.Txt_Pwd.Text);
               UDPComunicationClient.Start;
          end;
     end else begin
               if Chk_Remote_Control.Checked then
               begin
                    if UDPComunicationServer=nil then
                    begin
                         UDPComunicationServer     := TUDPComunicationServer.Create(IntToStr(port_s + i), Self.Txt_Pwd.Text);
                         UDPComunicationServer.SetValidConnection(Txt_Host.text, (port_r + i));
                         UDPComunicationServer.SetTimeOut(50);
                         UDPComunicationServer.Start;
                    end;
               end;
     end;


     if not Self.Chk_SendScreen.Checked then
     begin
         Self.Memo1.Append('Servers started');
         Self.Image1.Picture.Clear;
         Self.Img_OpenPopUp.Visible := true;
         Self.TabSheet2.show;
     end
     else
     begin
          Self.Memo1.Append('Clients started');

          if Assigned(MySmartTrayIcon) then
          begin
               MySmartTrayIcon.Show(Self.ImageList1,0,Self.Caption);
          end;

     end;

end;

procedure TFrmMain.Button2Click(Sender: TObject);
var
   app : string;
begin
     app                                  := Self.Txt_Port_Sending.Text;
     Self.Txt_Port_Sending.Text           := Self.Txt_Remote_Port_Reception.Text;
     Self.Txt_Remote_Port_Reception.Text  := app;
     Self.Chk_SendScreen.Checked          := not Self.Chk_SendScreen.Checked;
     Self.Chk_Jumbo.Checked               := not Self.Chk_Jumbo.Checked;
     Self.Chk_Remote_Control.Checked      := not Self.Chk_Remote_Control.Checked;
     Self.Chk_With_Mouse_Position.Checked := not Self.Chk_With_Mouse_Position.Checked;
end;

procedure TFrmMain.Button3Click(Sender: TObject);
var
   i : integer;
begin

     {$ifdef GetScreenInMainThread}
             ByPassTimer  := true;
     {$endif}

     for i:=0 to ((COUNT_ROW_ZONE_SCREEN*COUNT_COL_ZONE_SCREEN)-1) do
     begin
               if UDPClientThread1[i]<>nil then
               begin
                    UDPClientThread1[i].FreeOnTerminate:=true;
                    UDPClientThread1[i].Terminate;
                    UDPClientThread1[i]:=nil;
               end;
               if UDPServerThread1[i]<>nil then
               begin
                    UDPServerThread1[i].FreeOnTerminate:=true;
                    UDPServerThread1[i].Terminate;
                    UDPServerThread1[i]:=nil;
               end;
     end;

     if Assigned(UDPComunicationClient) then
     begin
          UDPComunicationClient.FreeOnTerminate:=true;
          UDPComunicationClient.Terminate;
          UDPComunicationClient:=nil;
     end;

     if Assigned(UDPComunicationServer) then
     begin
          UDPComunicationServer.FreeOnTerminate:=true;
          UDPComunicationServer.Terminate;
          UDPComunicationServer:=nil;
     end;

     Self.Memo1.Append('Stopped');

     Self.Image1.Picture.Clear;
     Self.Chk_SendScreen.Enabled               := true;
     Self.Chk_With_Mouse_Position.Enabled      := true;
     Self.TrackBar1.Enabled                    := true;
     Self.Txt_Port_Sending.Enabled             := true;
     Self.Txt_Remote_Port_Reception.Enabled    := true;
     Self.Txt_Host.Enabled                     := true;
     Chk_Jumbo.Enabled                         := true;
     Self.TrackBar2.Enabled                    := true;
     Self.Txt_Pwd.Enabled                      := true;
end;

procedure TFrmMain.Button4Click(Sender: TObject);
var
   i : integer;
begin
     StopUploadImage                    := true;
     for i := 0 to 100 do
     begin

          Self.Panel_PopUp.Width := Self.Panel_PopUp.Width - 1;
          Self.Panel_PopUp.Left  := Self.Panel_PopUp.Left + 1;

          if (i mod 10)=0 then
          begin
               Self.Panel_PopUp.Invalidate;
               Sleep(50);
               Application.ProcessMessages;
          end;

     end;

     Self.Panel_PopUp.Invalidate;
     Application.ProcessMessages;
     Sleep(50);

     Panel_PopUp.Visible    := false;

     if Self.TabSheet2.Showing then
        Self.Img_OpenPopUp.Visible := true;

     StopUploadImage := false;
end;

procedure TFrmMain.FormDestroy(Sender: TObject);
begin
     DoneCriticalsection(UDPCriticalSectionMouse);
     DoneCriticalsection(UDPCriticalSection);
     if Assigned(MySmartTrayIcon) then
     begin
          MySmartTrayIcon.Free;
          MySmartTrayIcon := nil;
     end;
     {$ifdef GetScreenInMainThread}
          MyScreen.Free;
          MyScreen := nil;
     {$endif}

     if Assigned(MemoryJpg) then
     begin
          MemoryJpg.Free;
          MemoryJpg:=nil;
     end;

end;

procedure TFrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

end;

procedure TFrmMain.FormKeyPress(Sender: TObject; var Key: char);
begin

end;

procedure TFrmMain.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
  );
begin
     if Self.TabSheet2.Showing then
     begin
          if not Self.Chk_SendScreen.Checked then
          begin
               LastKeyPressd    := IntToStr(Key);

               LastShiftPressed := '';

               if Self.Tbtn_Shift.Checked then
                   LastShiftPressed := LastShiftPressed + 'ssShift '
               else
                   if ssShift in Shift then
                      LastShiftPressed := LastShiftPressed + 'ssShift ';

               if Self.Tbtn_Alt.Checked then
                  LastShiftPressed := LastShiftPressed + 'ssAlt '
               else
                   if ssAlt in Shift then
                      LastShiftPressed := LastShiftPressed + 'ssAlt ';

               if Self.TBtn_Ctrl.Checked then
                  LastShiftPressed := LastShiftPressed + 'ssCtrl '
               else
                   if ssCtrl in Shift then
                      LastShiftPressed := LastShiftPressed + 'ssCtrl ';

               if Self.TBtn_AltGr.Checked then
                  LastShiftPressed := LastShiftPressed + 'ssAltGr '
               else
                   if ssAltGr in Shift then
                      LastShiftPressed := LastShiftPressed + 'ssAltGr ';

               if Self.TBtn_CapsLock.Checked then
               begin
                    LastShiftPressed := LastShiftPressed + 'ssCaps ';
               end else begin
                    if ssCaps in Shift then
                       LastShiftPressed := LastShiftPressed + 'ssCaps ';
               end;

               if Self.TBtn_Num.Checked then
                  LastShiftPressed := LastShiftPressed + 'ssNum '
               else
                   if ssNum in Shift then
                      LastShiftPressed := LastShiftPressed + 'ssNum ';

               LastShiftPressed := trim(LastShiftPressed);

               Key := 0;
          end;
     end;
end;

procedure TFrmMain.FormResize(Sender: TObject);
begin
     Self.Timer_Resize.Interval:=100;
end;

procedure TFrmMain.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
   MyX : integer;
   MyY : integer;
begin
     if not Self.Chk_SendScreen.Checked then
     begin
          MouseButton := 'MD';

          MyX := Trunc((X * Self.Image1.Picture.Jpeg.Width)/Self.Image1.Width);
          MyY := Trunc((Y * Self.Image1.Picture.Jpeg.Height)/Self.Image1.Height);

          Self.MouseMsg_Width    := Self.Image1.Picture.Jpeg.Width;
          Self.MouseMsg_Height   := Self.Image1.Picture.Jpeg.Height;
          Self.MouseMsg_X        := MyX;
          Self.MouseMsg_Y        := MyY;
          Self.MouseShift        := Shift;
     end;
end;

procedure TFrmMain.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
   MyX : integer;
   MyY : integer;
begin
     if not Self.Chk_SendScreen.Checked then
     begin
          if MouseMsg_Button = '' then
             MouseMsg_Button := 'MV';

          MyX := Trunc((X * Self.Image1.Picture.Jpeg.Width)/Self.Image1.Width);
          MyY := Trunc((Y * Self.Image1.Picture.Jpeg.Height)/Self.Image1.Height);

          Self.MouseMsg_Width    := Self.Image1.Picture.Jpeg.Width;
          Self.MouseMsg_Height   := Self.Image1.Picture.Jpeg.Height;
          Self.MouseMsg_X        := MyX;
          Self.MouseMsg_Y        := MyY;
          Self.MouseShift        := Shift;
     end;
end;

procedure TFrmMain.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
   MyX : integer;
   MyY : integer;
begin
     if not Self.Chk_SendScreen.Checked then
     begin

          MyX := Trunc((X * Self.Image1.Picture.Jpeg.Width)/Self.Image1.Width);
          MyY := Trunc((Y * Self.Image1.Picture.Jpeg.Height)/Self.Image1.Height);

          Self.MouseMsg_Width    := Self.Image1.Picture.Jpeg.Width;
          Self.MouseMsg_Height   := Self.Image1.Picture.Jpeg.Height;
          Self.MouseMsg_X        := MyX;
          Self.MouseMsg_Y        := MyY;
          Self.MouseShift        := Shift;

          Timer_Click.Interval := 0;
          Inc(NumClick);

          if Button = mbRight then
          begin
               MouseButton := 'RC';
          end else begin
               MouseButton:= 'LC';
          end;

          Timer_Click.Interval := 200;

     end;
end;

procedure TFrmMain.Image1Paint(Sender: TObject);
begin
     Image1.Canvas.Brush.Style := bsClear;
     Image1.Canvas.Pen.Color := clRed;
     Image1.Canvas.Rectangle(Image1.ClientRect);
end;

procedure TFrmMain.Image1Resize(Sender: TObject);
begin

end;

procedure TFrmMain.Img_OpenPopUpClick(Sender: TObject);
var
   i : integer;
begin
     StopUploadImage                          := true;
     Self.Panel_PopUp.Width                   := 1;
     Self.Panel_PopUp.Left                    := Self.Width - 1;
     Self.Panel_PopUp.Top                     := Self.Img_OpenPopUp.Top - Self.Panel_PopUp.Height - 4;


     Panel_PopUp.Visible                      := true;
     Self.Panel_PopUp.BringToFront;
     Self.Img_OpenPopUp.Visible               := false;

     for i := 0 to 100 do
     begin

          Self.Panel_PopUp.Width := Self.Panel_PopUp.Width + 1;
          Self.Panel_PopUp.Left  := Self.Panel_PopUp.Left - 1;

          if (i mod 10)=0 then
          begin
               Self.Panel_PopUp.Invalidate;
               Sleep(50);
               Self.Panel_PopUp.BringToFront;
               Application.ProcessMessages;
          end;

     end;
     StopUploadImage                       := false;
end;

procedure TFrmMain.MenuItem1Click(Sender: TObject);
begin
     if Assigned(MySmartTrayIcon) then
        MySmartTrayIcon.Hide;
end;

procedure TFrmMain.PageControl1Change(Sender: TObject);
begin
     if Self.TabSheet2.Showing then
        Img_OpenPopUp.visible := true
     else
         Img_OpenPopUp.visible := false;
end;

procedure TFrmMain.TBtn_CapsLockChange(Sender: TObject);
begin

end;

procedure TFrmMain.TBtn_CtrlChange(Sender: TObject);
begin

end;

procedure TFrmMain.Timer1Timer(Sender: TObject);
{$ifdef GetScreenInMainThread}
var
        sImg1           : TSmartDesktopImage;
{$endif}
begin
     Self.Timer1.Interval := 0;
     {$ifdef GetScreenInMainThread}
     if (not ByPassTimer) and (not Timer1_start) then
     begin
          Timer1_start   := true;
          sImg1           := TSmartDesktopImage.Create;
          sImg1.SetQuality(Self.TrackBar2.Position);
          sImg1.LoadScreen(MyScreen);

          MyScreenW := MyScreen.Width;
          MyScreenH := MyScreen.Height;

          if Self.Chk_With_Mouse_Position.Checked then
          begin
               MyScreen.Canvas.Brush.Color:= clRed;
               MyScreen.Canvas.Ellipse(Mouse.CursorPos.X, Mouse.CursorPos.Y, Mouse.CursorPos.X+Self.TrackBar1.Position, Mouse.CursorPos.Y+Self.TrackBar1.Position);
          end;

          sImg1.Free;
          sImg1 := nil;
          Timer1_start   := false;
     end;
     Self.Timer1.Interval := 10;
     {$endif}
end;

procedure TFrmMain.Timer_ClickTimer(Sender: TObject);
begin
     Timer_Click.Interval := 0;

     LastShiftPressed := '';

     if Self.Tbtn_Shift.Checked then
        LastShiftPressed := LastShiftPressed + 'ssShift '
     else
         if ssShift in MouseShift then
            LastShiftPressed := LastShiftPressed + 'ssShift ';

     if Self.Tbtn_Alt.checked then
         LastShiftPressed := LastShiftPressed + 'ssAlt '
     else
         if ssAlt in MouseShift then
            LastShiftPressed := LastShiftPressed + 'ssAlt ';

     if Self.TBtn_Ctrl.Checked then
         LastShiftPressed := LastShiftPressed + 'ssCtrl '
     else
         if ssCtrl in MouseShift then
            LastShiftPressed := LastShiftPressed + 'ssCtrl ';

     if Self.TBtn_AltGr.Checked then
         LastShiftPressed := LastShiftPressed + 'ssAltGr '
     else
         if ssAltGr in MouseShift then
            LastShiftPressed := LastShiftPressed + 'ssAltGr ';

     if Self.TBtn_CapsLock.Checked then
     begin
          LastShiftPressed := LastShiftPressed + 'ssCaps ';
     end else begin
          if ssCaps in MouseShift then
             LastShiftPressed := LastShiftPressed + 'ssCaps ';
     end;

     if Self.TBtn_Num.Checked then
        LastShiftPressed := LastShiftPressed + 'ssNum '
     else
         if ssNum in MouseShift then
            LastShiftPressed := LastShiftPressed + 'ssNum ';

     LastShiftPressed := trim(LastShiftPressed);

     if (NumClick=1) then
     begin
          Self.MouseMsg_Button := MouseButton;
     end else begin
          Self.MouseMsg_Button := 'DC';
     end;

     NumClick             := 0;
end;

procedure TFrmMain.Timer_ResizeTimer(Sender: TObject);
begin
     Self.Timer_Resize.Interval:=0;
     ScreenAdjusted := false;
     AdjustScreenImage;
end;

procedure TFrmMain.ClientNotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin

end;

procedure TFrmMain.ServerNotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin

end;

procedure TFrmMain.UDPServerShowImage;
var
   JpgS           : TJpegImage;
begin
     sImg               := TSmartDesktopImage.Create;
     JpgS               := TJpegImage.Create;

     MemoryJpg.Position := 0;
     JpgS.LoadFromStream(MemoryJpg);
     MemoryJpg.Clear;

     sImg.RefreshZoneOfScreenWithImage(JpgS, COUNT_ROW_ZONE_SCREEN, COUNT_COL_ZONE_SCREEN,(UDPServerIdx+1),Image1.Picture.Jpeg, RemoteScreenWidth, RemoteScreenHeight);
     JpgS.Free;
     JpgS := nil;
     sImg.Free;
     sImg := nil;

     AdjustScreenImage;
end;

procedure TFrmMain.UDPClientGetScreen;
{$ifdef GetScreenInMainThread}
var
        sImg1           : TSmartDesktopImage;
        JpgP            : TJpegImage;
{$endif}
begin
     {$ifdef GetScreenInMainThread}
     JpgP           := TJpegImage.Create;
     while (Self.Timer1_start) do
           Sleep(1);
     sImg1           := TSmartDesktopImage.Create;
     sImg1.CopyZoneOfScreenInOtherImage(MyScreen,COUNT_ROW_ZONE_SCREEN,COUNT_COL_ZONE_SCREEN,ThreadZone + 1,JpgP);
     ZoneB64 := sImg1.FromStreamToBase64(JpgP);
     sImg1.Free;
     sImg1 := nil;
     JpgP.Free;
     JpgP := nil;
     {$endif}
end;

procedure TFrmMain.UDPActionMouse;
var
   S_Height        : integer;
   S_Width         : integer;
   MyShift         : TShiftState;
   MyX             : integer;
   MyY             : integer;
begin

     Self.Txt_LastMouse_Values.Text := trim('X:' + IntToStr(MouseMsg_X) + ' Y:' + IntToStr(MouseMsg_Y) + ' ' + LastShiftPressed);

     S_Height        := Screen.Height;
     S_Width         := Screen.Width;

     MyY      := Trunc((MouseMsg_Height/S_Height) * MouseMsg_Y);
     MyX      := Trunc((MouseMsg_Width/S_Width) * MouseMsg_X);

     MyShift := [];
     if Pos('ssShift',LastShiftPressed)>0 then
        MyShift := MyShift + [ssShift];

     if Pos('ssAlt',LastShiftPressed)>0 then
        MyShift := MyShift + [ssAlt];

     if Pos('ssCtrl',LastShiftPressed)>0 then
        MyShift := MyShift + [ssCtrl];

     if Pos('ssAltGr',LastShiftPressed)>0 then
        MyShift := MyShift + [ssAltGr];

     if Pos('ssCaps',LastShiftPressed)>0 then
        MyShift := MyShift + [ssCaps];

     if Pos('ssNum',LastShiftPressed)>0 then
        MyShift := MyShift + [ssNum];

     case Self.MouseMsg_Button of

          'MV' : MouseInput.Move(MyShift,MyX, MyY);
          'DC' : MouseInput.DblClick(mbLeft, MyShift);
          'LC' : MouseInput.Click(mbLeft, MyShift);
          'RC' : MouseInput.Click(mbRight, MyShift);
          'MD' : MouseInput.Down(mbLeft, MyShift);

     end;

     Application.ProcessMessages;


end;

procedure TFrmMain.UDPActionKey;
var
   MyShift         : TShiftState;
   MyKey           : word;
begin

     Self.Txt_LastKey_Values.Text := Trim(Self.LastKeyPressd + ' ' + LastShiftPressed);

     MyKey := StrToInt(Self.LastKeyPressd);

     MyShift := [];
     if Pos('ssShift',LastShiftPressed)>0 then
        MyShift := MyShift + [ssShift];

     if Pos('ssAlt',LastShiftPressed)>0 then
        MyShift := MyShift + [ssAlt];

     if Pos('ssCtrl',LastShiftPressed)>0 then
        MyShift := MyShift + [ssCtrl];

     if Pos('ssAltGr',LastShiftPressed)>0 then
        MyShift := MyShift + [ssAltGr];

     if Pos('ssCaps',LastShiftPressed)>0 then
        MyShift := MyShift + [ssCaps];

     if Pos('ssNum',LastShiftPressed)>0 then
        MyShift := MyShift + [ssNum];

     KeyInput.Apply(MyShift);
     KeyInput.Press(MyKey);
     KeyInput.Unapply(MyShift);
     Application.ProcessMessages;
end;

procedure TFrmMain.AdjustScreenImage;
begin
     if not ScreenAdjusted then
     begin

          if Self.Image1.Picture.Jpeg.Height <= Self.Image1.Picture.Jpeg.Width then
          begin

               if Self.Image1.Picture.Jpeg.Height>0 then
                  Self.Image1.Width := trunc((Self.Image1.Height * Self.Image1.Picture.Jpeg.Width) / Self.Image1.Picture.Jpeg.Height);

          end else begin

               if Self.Image1.Picture.Jpeg.Width>0 then
                  Self.Image1.Height := trunc((Self.Image1.Width * Self.Image1.Picture.Jpeg.Height) / Self.Image1.Picture.Jpeg.Width);

          end;

          if Self.Image1.Width > Self.Image1.Picture.Jpeg.Width then
          begin
               Self.Image1.Width := Self.Image1.Picture.Jpeg.Width;
          end;

          if Self.Image1.Height > Self.Image1.Picture.Jpeg.Height then
          begin
               Self.Image1.Height := Self.Image1.Picture.Jpeg.Height;
          end;

          Self.Image1.Left := Trunc((Self.TabSheet2.Width-Self.Image1.Width)/2);
          Self.Image1.Top  := Trunc((Self.TabSheet2.Height-Self.Image1.Height)/2);

          Self.Image1.Invalidate;

          ScreenAdjusted := true;
     end;
end;

end.

