{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit UnitUDPTransmission;

{$mode objfpc}{$H+}
interface

uses
  Classes, SysUtils,dateutils, blcksock, strutils, uSmartEncrypt, Graphics, md5;



type

    TNGITUDPComponentServerGetString = procedure(Sender: TObject; const Value: String) of object;
    TNGITUDPComponentStatus          = procedure(Sender: TObject; const Reason: String; const Value: String) of object;
    TAskNotifyError                  = procedure(Sender: TObject; id_error : integer; desc_error : string) of object;


    { TNGITUDPComponent }

    TNGITUDPComponent = class
        private
               myport                             : string;
               password                           : string;
               Sock                               : TUDPBlockSocket;
               conn_ok                            : boolean;
               FAskNotifyError                    : TAskNotifyError;
               NextImage                          : TMemoryStream;

               procedure OnStatus(Sender: TObject; Reason: THookSocketReason;
                 const Value: string);

               {$ifdef SendWithMemory}
               procedure MyStringCompression(input: string; var output: TMemoryStream);
               function MyStringDecompression(var input: TMemoryStream) : string;
               {$endif}

        public
              constructor Create(myport_p: string; password_p: string);
              function IsStartedCorrect: boolean;
              destructor Free;
              procedure SendString(valore: string; remote_ip : string = ''; remote_port : integer = 0);
              function GetString(var remote_ip: string; var remote_port: integer; PacketTimeOut : integer): string;

              function HashString(value : string) : string;
              function SendStringWithResult(value: string; remote_ip : string = ''; remote_port : integer = 0) : boolean;
              function GetStringWithResult(remote_ip: string; remote_port: integer;
                var value: string): boolean;

              procedure SendMemory(MyMemory: TMemoryStream; remote_ip: string;
                remote_port: integer);
              procedure GetMemory(var remote_ip: string; var remote_port: integer; PacketTimeOut : integer; var MyMemory : TMemoryStream);
              function FromMemoryToHex(const s:TStream; Size : integer = 0; invert_direction : boolean = false) : string;
              procedure SendJpg(Jpg : TJPEGImage; MAX_PACKET_SIZE : integer = 8190; remote_ip : string = ''; remote_port : integer = 0);
              function GetJpg(var remote_ip: string; var remote_port: integer;
                Jpg: TJPEGImage; PacketTimeOut: integer): boolean;

        published
              property OnNotifyError: TAskNotifyError read FAskNotifyError write FAskNotifyError;
    end;


implementation
{$ifdef SendWithMemory}
uses
    ZStream;
{$endif}

{ TNGITUDPComponent }

procedure TNGITUDPComponent.OnStatus(Sender: TObject; Reason: THookSocketReason;
  const Value: string);
var
  sReason : String;
begin
  try
     try

        case Reason of
          HR_ResolvingBegin : sReason := 'HR_ResolvingBegin';
          HR_ResolvingEnd : sReason := 'HR_ResolvingEnd';
          HR_SocketCreate : sReason := 'HR_SocketCreate';
          HR_SocketClose : sReason := 'HR_SocketClose';
          HR_Bind : sReason := 'HR_Bind';
          HR_Connect : sReason := 'HR_Connect';
          HR_CanRead : sReason := 'HR_CanRead';
          HR_CanWrite : sReason := 'HR_CanWrite';
          HR_Listen : sReason := 'HR_Listen';
          HR_Accept : sReason := 'HR_Accept';
          HR_ReadCount : sReason := 'HR_ReadCount';
          HR_WriteCount : sReason := 'HR_WriteCount';
          HR_Wait : sReason := 'HR_Wait';
          HR_Error : sReason := 'HR_Error';
        end;

     finally

    end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

constructor TNGITUDPComponent.Create(myport_p: string; password_p: string);
begin
  try
     Self.myport          := myport_p;
     Self.password        := password_p;
     NextImage            := TMemoryStream.Create;

     conn_ok:=false;
     Sock:=TUDPBlockSocket.Create;
     try
        //On status show error and other
        //enable on status if you can more seen
        sock.OnStatus := @OnStatus;
        sock.CreateSocket;
        //create server
        sock.bind('0.0.0.0',myport);

        if sock.LastError<>0 then exit;

        conn_ok:=true;
     finally
     end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;


end;

procedure TNGITUDPComponent.SendString(valore: string; remote_ip: string;
  remote_port: integer);
{$ifdef SendWithMemory}
var
   app : TMemoryStream;
{$endif}
begin
  try
     try

        if conn_ok = true then
        begin

          sock.Connect( remote_ip , IntToStr(remote_port) );

          //if valore<>'' then
          //writeln('Invio - ', remote_ip , ':' , remote_port, '-->', Copy(valore,1,20));

          if Self.password<>'' then
             valore := Crypt(Self.password, valore);

          {$ifdef SendWithMemory}
          MyStringCompression(valore, app);

          app.Position := 0;
          sock.SendStream(app);

          if Assigned(app) then
          begin
               app.Free;
               app := nil;
          end;
          {$else}
          sock.SendString(valore);
          {$endif}

        end;

     finally
     end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

function TNGITUDPComponent.GetString(var remote_ip: string;
  var remote_port: integer; PacketTimeOut: integer): string;
var
   ret    : string;
   {$ifdef SendWithMemory}
   app    : TMemoryStream;
   {$endif}
begin
  try
     try

        {$ifdef SendWithMemory}
        app         := TMemoryStream.Create;


        sock.RecvStream(app, PacketTimeOut);

        if app.Size>0 then
        begin
             ret := MyStringDecompression(app);
        end;
        if Assigned(app) then
        begin
             app.Free;
             app := nil;
        end;
        {$else}
        ret         := sock.RecvPacket(PacketTimeOut);
        {$endif}

        if Self.password<>'' then
           ret := DeCrypt(Self.password, ret);

        remote_ip   := sock.GetRemoteSinIP;
        remote_port := sock.GetRemoteSinPort;


        //writeln('Ricevo - ', remote_ip , ':' , remote_port, '-->', Copy(ret,1,20));

        result:=ret;

     finally

    end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

function TNGITUDPComponent.HashString(value: string): string;
begin
     result := MD5Print(MD5String(value));
end;

function TNGITUDPComponent.SendStringWithResult(value: string;
  remote_ip: string; remote_port: integer): boolean;
var
   ret      : boolean;
   myhash   : string;
   getValue : string;
   r_ip     : string;
   r_port   : integer;
begin

     ret := false;

     myhash := Self.HashString(value);
     Self.SendString(value, remote_ip, remote_port);

     getValue := Self.GetString(r_ip, r_port,100);
     {$ifdef FilterFromIpPort}
          if (R_IP <> remote_ip) or (R_Port <> remote_port) then
          begin
               getValue := '';
          end;
     {$endif}
     if getValue = myhash then
     begin
          Self.SendString(ReverseString(myhash), remote_ip, remote_port);
          getValue := Self.GetString(r_ip, r_port,100);
          {$ifdef FilterFromIpPort}
               if (R_IP <> remote_ip) or (R_Port <> remote_port) then
               begin
                    getValue := '';
               end;
          {$endif}

          if getValue = 'END_' + myhash then
          begin
               Self.SendString('END_' + ReverseString(myhash), remote_ip, remote_port);
               ret := true;
          end;

     end;
     result := ret;
end;

function TNGITUDPComponent.GetStringWithResult(remote_ip: string;
  remote_port: integer; var value: string): boolean;
var
   ret      : boolean;
   myhash   : string;
   getValue : string;
   r_ip     : string;
   r_port   : integer;
begin

     ret   := false;
     value := '';

     getValue := Self.GetString(r_ip, r_port,100);
     {$ifdef FilterFromIpPort}
          if (R_IP <> remote_ip) or (R_Port <> remote_port) then
          begin
               getValue := '';
          end;
     {$endif}
     if getValue <> '' then
     begin
          value  := GetValue;
          myhash := Self.HashString(value);
          Self.SendString(myhash, remote_ip, remote_port);
          getValue := Self.GetString(r_ip, r_port,100);
          {$ifdef FilterFromIpPort}
               if (R_IP <> remote_ip) or (R_Port <> remote_port) then
               begin
                    getValue := '';
               end;
          {$endif}

          if getValue = ReverseString(myhash) then
          begin
               Self.SendString('END_' + myhash, remote_ip, remote_port);
               ret := true;
          end;

     end;
     if not ret then
        value := '';
     result := ret;
end;

procedure TNGITUDPComponent.SendMemory(MyMemory: TMemoryStream;
  remote_ip: string; remote_port: integer);
var
   appStr      : string;
begin
  try
     try

        if conn_ok = true then
        begin

          sock.Connect( remote_ip , IntToStr(remote_port) );


          if Self.password<>'' then
          begin
               MyMemory.Position:=0;
               appStr := MemoryStreamToStringDef(MyMemory,'');

               appStr := Crypt(Self.password,appStr);

               sock.SendString(appStr);

          end else begin
               MyMemory.Position := 0;
               sock.SendStream(MyMemory);
          end;


        end;

     finally
     end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

procedure TNGITUDPComponent.GetMemory(var remote_ip: string;
  var remote_port: integer; PacketTimeOut: integer; var MyMemory: TMemoryStream
  );
var
   appStr      : string;
begin
  try
     try


        MyMemory.Clear;

        if Self.password<>'' then
        begin
             appStr := sock.RecvPacket(PacketTimeOut);

             appStr := DeCrypt(Self.password,appStr);

             if not TryMemoryStreamFromString(MyMemory, appStr) then
             begin
                  MyMemory.Clear;
             end;
             MyMemory.Position := 0;
        end else begin
             MyMemory.Position := 0;
             sock.RecvStream(MyMemory, PacketTimeOut);
        end;

        remote_ip   := sock.GetRemoteSinIP;
        remote_port := sock.GetRemoteSinPort;


     finally

    end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

function TNGITUDPComponent.FromMemoryToHex(const s: TStream; Size: integer;
  invert_direction: boolean): string;
var
   ret    : string;
   i      : integer;
   k      : integer;
   exit   : boolean;
begin
     try
        try
             ret  := '';
             exit := false;

             if (not invert_direction) then
             begin
                  i    := 0;
                  while (i<=pred(s.Size)) and (not exit) do
                  begin
                       ret := ret + s.readbyte.ToHexString;
                       Inc(i);
                       if (Size>0)and(i=Size) then
                          exit:=true;
                  end;

             end else begin
                  i      := pred(s.Size);
                  k      := 0;
                  while (i>=0) and (not exit) do
                  begin
                       s.Position := i;
                       ret := s.readbyte.ToHexString + ret;
                       Dec(i);
                       Inc(k);
                       if (Size>0)and(k=Size) then
                          exit:=true;
                  end;
             end;

             result := ret;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TNGITUDPComponent.SendJpg(Jpg: TJPEGImage; MAX_PACKET_SIZE: integer;
  remote_ip: string; remote_port: integer);
var
   Mem               : TMemoryStream;
   Pak               : TMemoryStream;
   i                 : integer;
   loopend           : integer;
   tot_sended        : integer;
begin

     try
        try

                  Mem := TMemoryStream.Create;
                  Jpg.SaveToStream(Mem);
                  Mem.Position := 0;

                  i       := 0;
                  loopend := Mem.Size;
                  tot_sended := 0;

                  while (i<loopend) do
                  begin
                       Pak  := TMemoryStream.Create;

                       if (tot_sended + MAX_PACKET_SIZE)<loopend then
                       begin
                          Pak.CopyFrom(Mem, MAX_PACKET_SIZE);
                          tot_sended := tot_sended + MAX_PACKET_SIZE;
                       end
                       else
                       begin
                          Pak.CopyFrom(Mem, loopend - tot_sended);
                          tot_sended := tot_sended + (loopend - tot_sended);
                       end;

                       Pak.Position:=0;

                       Mem.Position := tot_sended;

                       Self.SendMemory(Pak,remote_ip,remote_port);

                       //Sleep(4);

                       Pak.Free;
                       Pak := nil;

                       i := i + MAX_PACKET_SIZE;
                  end;

                  Mem.Free;
                  Mem:=nil;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TNGITUDPComponent.GetJpg(var remote_ip: string;
  var remote_port: integer; Jpg: TJPEGImage; PacketTimeOut: integer
  ): boolean;
var
   i               : integer;
   r_ip            : string;
   r_port          : integer;
   MyMem           : TMemoryStream;
   Posiz           : integer;
   SignaturePkg    : string;
   EndPkg          : string;
   ExitLoop        : boolean;
   StartDate       : TDateTime;
begin
     StartDate   := now();
     remote_ip   := '';
     remote_port := 0;
     result      := false;

     NextImage.Clear;
     NextImage.Position := 0;

     MyMem     := TMemoryStream.Create;
     ExitLoop  := false;
     i := 0;
     while (not ExitLoop) do
     begin

          Self.GetMemory (r_ip, r_port, 100, MyMem);
          MyMem.Position := 0;

          SignaturePkg := FromMemoryToHex (MyMem,12);
          if SignaturePkg = 'FFD8FFE000104A4649460001' then
          begin
             NextImage.Clear;
             NextImage.Position := 0;
          end;
          MyMem.Position := 0;
          EndPkg         := FromMemoryToHex (MyMem,2,true);
          MyMem.Position := 0;

          MyMem.SaveToStream(NextImage);

          if EndPkg = 'FFD9' then
          begin

             if MyMem.Size>0 then
             begin

                try
                    try

                        Posiz              := NextImage.Position;
                        NextImage.Position := 0;

                        Jpg.LoadFromStream(NextImage);
                        NextImage.Clear;

                        remote_ip   := r_ip;
                        remote_port := r_port;
                        result      := true;
                        ExitLoop    := true;
                    finally
                           NextImage.Position := Posiz;
                    end;
                 except
                       on E: Exception do
                       begin
                            Jpg.Clear;
                            {$ifdef WriteDebug}
                            writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                            {$endif}
                       end;
                 end;
             end;
          end else begin
              if MyMem.Size>0 then
                 StartDate := Now();
          end;

          if not ExitLoop then
          begin
             //Sleep(2);

             if Abs(MilliSecondsBetween(Now(),StartDate))>=PacketTimeOut then
                ExitLoop := true;
          end;

     end;
     if Assigned(MyMem) then
     begin
        MyMem.Free;
        MyMem:=nil;
     end;
end;

{$ifdef SendWithMemory}
procedure TNGITUDPComponent.MyStringCompression(input: string;
  var output: TMemoryStream);
var
   {$ifdef SendWithCompression}
   compressor : Tcompressionstream;
   {$endif}
   app        : TStringStream;
begin

     try
        try

           output       := TMemoryStream.Create;
           app          := TStringStream.Create(input);
           app.Position := 0;
           {$ifdef SendWithCompression}
           compressor   := Tcompressionstream.create(clmax, output);
           compressor.CopyFrom(app, app.Size);
           {$else}
           output.CopyFrom(app, app.Size);
           {$endif}
        finally
          {$ifdef SendWithCompression}
          if Assigned(compressor) then
          begin
               compressor.Free;
               compressor := nil;
          end;
          {$endif}

          if Assigned(app) then
          begin
               app.Free;
               app := nil;
          end;

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}

           end;
     end;
end;
{$endif}

{$ifdef SendWithMemory}
function TNGITUDPComponent.MyStringDecompression(var input: TMemoryStream
  ): string;
var
   {$ifdef SendWithCompression}
   decompressor   : TDecompressionStream;
   {$endif}
   ret            : string;
   app            : TStringStream;
begin
     try
        try
           ret            := '';
           input.Position := 0;

           app            := TStringStream.Create('');
           app.Position   := 0;
           app.Size       := 0;
           {$ifdef SendWithCompression}
           decompressor   := TDecompressionStream.create(input);
           app.CopyFrom(decompressor, 0);
           Decompressor.Free;
           {$else}
           app.CopyFrom(input, 0);
           {$endif}

           ret    := app.DataString;

        finally

          if Assigned(app) then
          begin
               app.Free;
               app := nil;
          end;

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}

           end;
     end;

     result := ret;
end;
{$endif}

function TNGITUDPComponent.IsStartedCorrect: boolean;
begin
  try
     try

        result:=conn_ok;

     finally

    end;
  except
        on E: Exception do
        begin

             {$ifdef WriteDebug}
             writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
             {$endif}
        end;
  end;
end;

destructor TNGITUDPComponent.Free;
begin
     if Assigned(NextImage) then
     begin
          NextImage.Free;
          NextImage := nil;
     end;

     try
        try
           if conn_ok = true then
              sock.CloseSocket;
        finally
               sock.free;
        end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TNGITUDPComponent->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

end.

