{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uSmartDesktopImage;

{$mode objfpc}{$H+}

{$ifdef Unix}
       {$ifdef Darwin}
       {$else}
              //{$define BypassBeginUpdate}
              {$define ByPasspf32bit}
       {$endif}
{$endif}

interface

uses
  Classes, SysUtils, Graphics, lclintf, LCLType, function_base64;

type

    { TSmartDesktopImage }

    TSmartDesktopImage = class
          private
                 FQuality  : integer;
                 procedure GetDesktopImage(MyImg: TJpegImage); dynamic;
                 procedure GetDesktopImage(MyImg: TBitmap); dynamic;

                 function CompPixel(pixel1 : byte; pixel2 : byte) : boolean;
          public
                constructor Create;
                destructor Free;
                procedure LoadScreen(MyImg: TJpegImage); dynamic;
                procedure LoadScreen(MyImg: TBitmap); dynamic;

                procedure CopyScreenInOtherImage(source: TJpegImage; destination: TJpegImage); dynamic;
                procedure CopyScreenInOtherImage(source: TBitmap; destination: TBitmap); dynamic;

                procedure CopyZoneOfScreenInOtherImage(source: TJpegImage; NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer; destination: TJpegImage); dynamic;
                procedure CopyZoneOfScreenInOtherImage(source: TBitmap; NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer; destination: TBitmap); dynamic;

                procedure RefreshZoneOfScreenWithImage(source: TJpegImage; NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer; destination: TJpegImage; dWidth : integer; dHeight: integer); dynamic;
                procedure RefreshZoneOfScreenWithImage(source: TBitmap; NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer; destination: TBitmap; dWidth : integer; dHeight: integer); dynamic;

                procedure CalculateSizeZone(img_width: integer; img_height : integer; NumbersOfXZone: integer; NumbersOfYZone: integer; var width: integer; var height: integer);

                function GetQuality() : integer;
                procedure SetQuality(value : integer);

                function CompareImgages(source: TJpegImage; destination: TJpegImage) : integer; dynamic;
                function CompareImgages(source: TBitmap; destination: TBitmap) : integer; dynamic;

                function  FromStreamToBase64(MyImg: TJpegImage) : string; dynamic;
                function  FromStreamToBase64(MyImg: TBitmap) : string; dynamic;

                procedure FromBase64ToStream(MyImg: TJpegImage; sb64 : string); dynamic;
                procedure FromBase64ToStream(MyImg: TBitmap; sb64 : string); dynamic;

                procedure Jpeg2Bmp(Jpg : TJpegImage; Bmp : TBitmap);
                procedure Bmp2Jpeg(Bmp : TBitmap; Jpg : TJpegImage);
    end;

implementation




{ TSmartDesktopImage }

procedure TSmartDesktopImage.GetDesktopImage(MyImg: TJpegImage);
var
  ScreenDC   : HDC;
begin
     if Assigned(MyImg) then
     begin
       ScreenDC   := GetDC(0);
       try
          try
             {$ifndef ByPasspf32bit}
             MyImg.PixelFormat := pf32bit;
             {$endif}
             MyImg.LoadFromDevice(ScreenDC);
             MyImg.CompressionQuality:=Self.FQuality; //100 = best quality, 25 = pretty awful
          finally
            ReleaseDC(0,ScreenDC);
         end;
       except
             on E: Exception do
             begin
                  {$ifdef WriteDebug}
                  writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                  {$endif}
             end;
       end;
     end;
end;

procedure TSmartDesktopImage.GetDesktopImage(MyImg: TBitmap);
var
  ScreenDC   : HDC;
begin
     if Assigned(MyImg) then
     begin
       ScreenDC   := GetDC(0);
       try
          try
            {$ifndef ByPasspf32bit}
            MyImg.PixelFormat := pf32bit;
            {$endif}
            MyImg.LoadFromDevice(ScreenDC);
          finally
            ReleaseDC(0,ScreenDC);
         end;
       except
             on E: Exception do
             begin
                  {$ifdef WriteDebug}
                  writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                  {$endif}
             end;
       end;
     end;
end;

function TSmartDesktopImage.CompPixel(pixel1: byte; pixel2: byte): boolean;
var
   ret : boolean;
begin
     try
        try

          ret := false;

          if Abs(pixel2-pixel1)<=4 then
             ret := true;

          result := ret;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.Jpeg2Bmp(Jpg: TJpegImage; Bmp: TBitmap);
begin
     try
       if Assigned(Jpg) and Assigned(Bmp) then
       begin
            {$ifndef ByPasspf32bit}
            Bmp.PixelFormat := pf32bit;
            {$endif}
            try

              Bmp.Assign(Jpg);

            finally

            end;
       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.Bmp2Jpeg(Bmp: TBitmap; Jpg: TJpegImage);
begin
     try
       if Assigned(Bmp) and Assigned(Jpg) then
       begin
            {$ifndef ByPasspf32bit}
            Jpg.PixelFormat := pf32bit;
            {$endif}
            try

              Jpg.Assign(Bmp);
              Jpg.CompressionQuality := Self.FQuality;

            finally
            end;

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

constructor TSmartDesktopImage.Create;
begin
     try
        try

           FQuality := 25;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

destructor TSmartDesktopImage.Free;
begin
     try
        try



        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.LoadScreen(MyImg: TJpegImage);
begin
     try
        try

           Self.GetDesktopImage(MyImg);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.LoadScreen(MyImg: TBitmap);
begin
     try
        try

           Self.GetDesktopImage(MyImg);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.CopyScreenInOtherImage(source: TJpegImage;
  destination: TJpegImage);
var
   x  : integer;
   y  : integer;
   p1 : PRGBQuad;
   p2 : PRGBQuad;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
            destination.Clear;
            {$ifndef BypassBeginUpdate}
            destination.BeginUpdate();
            {$endif}
            {$ifndef ByPasspf32bit}
            destination.PixelFormat := pf32bit;
            {$endif}
            destination.SetSize(source.Width, source.Height);

            for y := 0 to source.Height - 1 do
            begin
                 p1 := source.Scanline[y];
                 p2 := destination.Scanline[y];
                 for x := 0 to source.Width - 1 do
                 begin
                      p2^.rgbRed            := p1^.rgbRed;
                      p2^.rgbGreen          := p1^.rgbGreen;
                      p2^.rgbBlue           := p1^.rgbBlue;
                      p2^.rgbReserved       := p1^.rgbReserved;
                      Inc(p1);
                      Inc(p2);
                 end;
            end;
            {$ifndef BypassBeginUpdate}
            destination.EndUpdate();
            {$endif}
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.CopyScreenInOtherImage(source: TBitmap;
  destination: TBitmap);
var
   x  : integer;
   y  : integer;
   p1 : PRGBQuad;
   p2 : PRGBQuad;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
            destination.Clear;
            {$ifndef BypassBeginUpdate}
            destination.BeginUpdate();
            {$endif}
            {$ifndef ByPasspf32bit}
            destination.PixelFormat := pf32bit;
            {$endif}
            destination.SetSize(source.Width+1, source.Height+1);

            for y := 0 to source.Height - 1 do
            begin
                 p1 := source.Scanline[y];
                 p2 := destination.Scanline[y];
                 for x := 0 to source.Width - 1 do
                 begin
                      p2^.rgbRed            := p1^.rgbRed;
                      p2^.rgbGreen          := p1^.rgbGreen;
                      p2^.rgbBlue           := p1^.rgbBlue;
                      p2^.rgbReserved       := p1^.rgbReserved;
                      Inc(p1);
                      Inc(p2);
                 end;
            end;
            {$ifndef BypassBeginUpdate}
            destination.EndUpdate();
            {$endif}
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

//NumbersOfXZone = numero di zone sulla larghezza
//NumbersOfYZone = numero di zone sull'altezza
//Zone parte da 1
//Ipotizziamo di usare 4 zone l'ordine sarà come segue
// 1 3
// 2 4
procedure TSmartDesktopImage.CopyZoneOfScreenInOtherImage(source: TJpegImage; NumbersOfXZone : integer; NumbersOfYZone : integer;
  Zone: integer; destination: TJpegImage);
var
   sWidth  : integer;
   sHeight : integer;
   i       : integer;
   j       : integer;
   k       : integer;
   {$ifdef MyVersion}
   x       : integer;
   y       : integer;
   p1      : PRGBQuad;
   p2      : PRGBQuad;
   y2      : integer;
   XEnd    : integer;
   YEnd    : integer;
   {$endif}
   exit    : boolean;
   XStart  : integer;
   YStart  : integer;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
               destination.Clear;

               {$ifndef MyVersion}

               Self.CalculateSizeZone(source.Width, source.Height, NumbersOfXZone, NumbersOfYZone, sWidth, sHeight);
               destination.SetSize(sWidth + 1, sHeight + 1);
               i    := 1;
               k    := 1;
               exit := false;
               while(i<=NumbersOfXZone)and(not exit) do
               begin
                    j := 1;
                    while(j<=NumbersOfYZone)and(not exit) do
                    begin
                         if (Zone = k) then
                         begin
                              XStart := sWidth * (i - 1);
                              YStart := sHeight * (j - 1);
                              exit := true;
                         end else begin
                             Inc(k);
                             Inc(j);
                         end;
                    end;
                    Inc(i);
               end;
               BitBlt(destination.Canvas.Handle, 0, 0, destination.Width, destination.Height, source.Canvas.Handle, XStart, YStart, cmSrcCopy);
               {$else}
               {$ifndef BypassBeginUpdate}
               destination.BeginUpdate();
               {$endif}
               {$ifndef ByPasspf32bit}
               destination.PixelFormat := pf32bit;
               {$endif}
               Self.CalculateSizeZone(source.Width, source.Height, NumbersOfXZone, NumbersOfYZone, sWidth, sHeight);
               i    := 1;
               k    := 1;
               exit := false;

               while(i<=NumbersOfXZone)and(not exit) do
               begin
                    j := 1;
                    while(j<=NumbersOfYZone)and(not exit) do
                    begin
                         if (Zone = k) then
                         begin
                              //destination.SetSize(source.Width, source.Height);
                              destination.SetSize(sWidth + 1, sHeight + 1);
                              YEnd  := sHeight*j;
                              if (YEnd>source.Height - 1) then
                                  YEnd := source.Height - 1;
                              y2 := 0;
                              for y := ((sHeight*j)-sHeight) to YEnd do
                              begin
                                   p1      := source.Scanline[y];
                                   p2      := destination.Scanline[y2];
                                   XEnd    := sWidth*i;
                                   if (XEnd>source.Width - 1) then
                                      XEnd := source.Width - 1;
                                   for x := 0 to XEnd do
                                   begin
                                        if (x>=((sWidth*i)-sWidth)) then
                                        begin
                                          p2^.rgbRed            := p1^.rgbRed;
                                          p2^.rgbGreen          := p1^.rgbGreen;
                                          p2^.rgbBlue           := p1^.rgbBlue;
                                          p2^.rgbReserved       := p1^.rgbReserved;
                                          Inc(p2);
                                        end;
                                        Inc(p1);
                                   end;
                                   Inc(y2);
                              end;

                              exit := true;
                         end else begin
                              Inc(k);
                              Inc(j);
                         end;
                    end;
                    Inc(i);
               end;
               {$ifndef BypassBeginUpdate}
               destination.EndUpdate();
               {$endif}

               {$endif}

          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.CopyZoneOfScreenInOtherImage(source: TBitmap;
  NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer;
  destination: TBitmap);
var
   sWidth  : integer;
   sHeight : integer;
   i       : integer;
   j       : integer;
   k       : integer;
   x       : integer;
   y       : integer;
   p1      : PRGBQuad;
   p2      : PRGBQuad;
   y2      : integer;
   exit    : boolean;
   XEnd    : integer;
   YEnd    : integer;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
               destination.Clear;
               {$ifndef BypassBeginUpdate}
               destination.BeginUpdate();
               {$endif}
               {$ifndef ByPasspf32bit}
               destination.PixelFormat := pf32bit;
               {$endif}
               Self.CalculateSizeZone(source.Width, source.Height, NumbersOfXZone, NumbersOfYZone, sWidth, sHeight);
               i    := 1;
               k    := 1;
               exit := false;

               while(i<=NumbersOfXZone)and(not exit) do
               begin
                    j := 1;
                    while(j<=NumbersOfYZone)and(not exit) do
                    begin
                         if (Zone = k) then
                         begin
                              //destination.SetSize(source.Width, source.Height);
                              destination.SetSize(sWidth + 1, sHeight + 1);
                              YEnd  := sHeight*j;
                              if (YEnd>source.Height - 1) then
                                  YEnd := source.Height - 1;
                              y2 := 0;
                              for y := ((sHeight*j)-sHeight) to YEnd do
                              begin
                                   p1      := source.Scanline[y];
                                   p2      := destination.Scanline[y2];
                                   XEnd    := sWidth*i;
                                   if (XEnd>source.Width - 1) then
                                      XEnd := source.Width - 1;
                                   for x := 0 to XEnd do
                                   begin
                                        if (x>=((sWidth*i)-sWidth)) then
                                        begin
                                          p2^.rgbRed            := p1^.rgbRed;
                                          p2^.rgbGreen          := p1^.rgbGreen;
                                          p2^.rgbBlue           := p1^.rgbBlue;
                                          p2^.rgbReserved       := p1^.rgbReserved;
                                          Inc(p2);
                                        end;
                                        Inc(p1);
                                   end;
                                   Inc(y2);
                              end;

                              exit := true;
                         end else begin
                              Inc(k);
                              Inc(j);
                         end;
                    end;
                    Inc(i);
               end;
               {$ifndef BypassBeginUpdate}
               destination.EndUpdate();
               {$endif}
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.RefreshZoneOfScreenWithImage(source: TJpegImage;
  NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer;
  destination: TJpegImage; dWidth: integer; dHeight: integer);
var
   sWidth  : integer;
   sHeight : integer;
   i       : integer;
   j       : integer;
   k       : integer;
   x       : integer;
   y       : integer;
   p1      : PRGBQuad;
   p2      : PRGBQuad;
   y2      : integer;
   exit    : boolean;
   XEnd    : integer;
   YEnd    : integer;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
               {$ifndef BypassBeginUpdate}
               destination.BeginUpdate();
               {$endif}
               {$ifndef ByPasspf32bit}
               destination.PixelFormat := pf32bit;
               {$endif}
               Self.CalculateSizeZone(dWidth, dHeight, NumbersOfXZone, NumbersOfYZone, sWidth, sHeight);
               destination.SetSize(dWidth, dHeight);
               i    := 1;
               k    := 1;
               exit := false;

               while(i<=NumbersOfXZone)and(not exit) do
               begin
                    j := 1;
                    while(j<=NumbersOfYZone)and(not exit) do
                    begin
                         if (Zone = k) then
                         begin
                              YEnd  := sHeight*j;
                              if (YEnd>dHeight - 1) then
                                  YEnd := dHeight - 1;
                              y2 := 0;
                              for y := ((sHeight*j)-sHeight) to YEnd do
                              begin
                                   p1      := destination.Scanline[y];
                                   p2      := source.Scanline[y2];
                                   XEnd    := sWidth*i;
                                   if (XEnd>dWidth - 1) then
                                      XEnd := dWidth - 1;
                                   for x := 0 to XEnd do
                                   begin
                                        if (x>=((sWidth*i)-sWidth)) then
                                        begin
                                          p1^.rgbRed            := p2^.rgbRed;
                                          p1^.rgbGreen          := p2^.rgbGreen;
                                          p1^.rgbBlue           := p2^.rgbBlue;
                                          p1^.rgbReserved       := p2^.rgbReserved;
                                          Inc(p2);
                                        end;
                                        Inc(p1);
                                   end;
                                   Inc(y2);
                              end;

                              exit := true;
                         end else begin
                              Inc(k);
                              Inc(j);
                         end;
                    end;
                    Inc(i);
               end;
               {$ifndef BypassBeginUpdate}
               destination.EndUpdate();
               {$endif}
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.RefreshZoneOfScreenWithImage(source: TBitmap;
  NumbersOfXZone: integer; NumbersOfYZone: integer; Zone: integer;
  destination: TBitmap; dWidth: integer; dHeight: integer);
var
   sWidth  : integer;
   sHeight : integer;
   i       : integer;
   j       : integer;
   k       : integer;
   x       : integer;
   y       : integer;
   p1      : PRGBQuad;
   p2      : PRGBQuad;
   y2      : integer;
   exit    : boolean;
   XEnd    : integer;
   YEnd    : integer;
begin
     try
        try

          if Assigned(source) and Assigned(destination) then
          begin
               {$ifndef BypassBeginUpdate}
               destination.BeginUpdate();
               {$endif}
               {$ifndef ByPasspf32bit}
               destination.PixelFormat := pf32bit;
               {$endif}
               Self.CalculateSizeZone(dWidth, dHeight, NumbersOfXZone, NumbersOfYZone, sWidth, sHeight);
               destination.SetSize(dWidth, dHeight);
               i    := 1;
               k    := 1;
               exit := false;

               while(i<=NumbersOfXZone)and(not exit) do
               begin
                    j := 1;
                    while(j<=NumbersOfYZone)and(not exit) do
                    begin
                         if (Zone = k) then
                         begin
                              YEnd  := sHeight*j;
                              if (YEnd>dHeight - 1) then
                                  YEnd := dHeight - 1;
                              y2 := 0;
                              for y := ((sHeight*j)-sHeight) to YEnd do
                              begin
                                   p1      := destination.Scanline[y];
                                   p2      := source.Scanline[y2];
                                   XEnd    := sWidth*i;
                                   if (XEnd>dWidth - 1) then
                                      XEnd := dWidth - 1;
                                   for x := 0 to XEnd do
                                   begin
                                        if (x>=((sWidth*i)-sWidth)) then
                                        begin
                                          p1^.rgbRed            := p2^.rgbRed;
                                          p1^.rgbGreen          := p2^.rgbGreen;
                                          p1^.rgbBlue           := p2^.rgbBlue;
                                          p1^.rgbReserved       := p2^.rgbReserved;
                                          Inc(p2);
                                        end;
                                        Inc(p1);
                                   end;
                                   Inc(y2);
                              end;

                              exit := true;
                         end else begin
                              Inc(k);
                              Inc(j);
                         end;
                    end;
                    Inc(i);
               end;
               {$ifndef BypassBeginUpdate}
               destination.EndUpdate();
               {$endif}
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

//NumbersOfXZone = numero di zone sulla larghezza
//NumbersOfYZone = numero di zone sull'altezza
procedure TSmartDesktopImage.CalculateSizeZone(img_width: integer;
  img_height: integer; NumbersOfXZone: integer; NumbersOfYZone: integer;
  var width: integer; var height: integer);
begin
     try
        try

          if ((img_width>0) and (img_height>0)) then
          begin
               width  := Trunc(img_width / NumbersOfXZone);
               height := Trunc(img_height / NumbersOfYZone);
          end else begin
               width  := 0;
               height := 0;
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TSmartDesktopImage.GetQuality(): integer;
begin
     try
        try

           result := Self.FQuality;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.SetQuality(value: integer);
begin
     try
        try

          if value < 1 then
          begin
               Self.FQuality := 1;
          end else begin
               if value > 100 then
               begin
                    Self.FQuality := 100;
               end else begin
                    Self.FQuality := value;
               end;
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TSmartDesktopImage.CompareImgages(source: TJpegImage;
  destination: TJpegImage): integer;
var
   x   : integer;
   y   : integer;
   p1  : PRGBQuad;
   p2  : PRGBQuad;
   tot : integer;
   qta : integer;
   ret : integer;
begin
     try
        try

          ret := 0;
          if Assigned(source) and Assigned(destination) then
          begin

               if (source.Width=destination.Width) and (source.Height=destination.Height) then
               begin

                 tot := 0;
                 qta := 0;

                 for y := 0 to source.Height - 1 do
                 begin
                      p1 := source.Scanline[y];
                      p2 := destination.Scanline[y];
                      for x := 0 to source.Width - 1 do
                      begin
                           if (CompPixel(p2^.rgbRed,p1^.rgbRed)) and (CompPixel(p2^.rgbGreen,p1^.rgbGreen)) and (CompPixel(p2^.rgbBlue, p1^.rgbBlue)) then
                           begin
                                inc(qta);
                           end;
                           Inc(p1);
                           Inc(p2);
                           Inc(tot);
                      end;
                 end;

                 ret := Trunc((100*qta)/tot);

               end;


          end;

        finally
               result := ret;
       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TSmartDesktopImage.CompareImgages(source: TBitmap; destination: TBitmap
  ): integer;
var
   x   : integer;
   y   : integer;
   p1  : PRGBQuad;
   p2  : PRGBQuad;
   tot : integer;
   qta : integer;
   ret : integer;
begin
     try
        try

          ret := 0;
          if Assigned(source) and Assigned(destination) then
          begin

               if (source.Width=destination.Width) and (source.Height=destination.Height) then
               begin

                 tot := 0;
                 qta := 0;

                 for y := 0 to source.Height - 1 do
                 begin
                      p1 := source.Scanline[y];
                      p2 := destination.Scanline[y];
                      for x := 0 to source.Width - 1 do
                      begin
                           if (CompPixel(p2^.rgbRed,p1^.rgbRed)) and (CompPixel(p2^.rgbGreen,p1^.rgbGreen)) and (CompPixel(p2^.rgbBlue, p1^.rgbBlue)) then
                           begin
                                inc(qta);
                           end;
                           Inc(p1);
                           Inc(p2);
                           Inc(tot);
                      end;
                 end;

                 ret := Trunc((100*qta)/tot);

               end;


          end;



        finally
               result := ret;
       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TSmartDesktopImage.FromStreamToBase64(MyImg: TJpegImage): string;
var
   ret : string;
   MS  : TMemoryStream;
begin
     try
        try

          ret := '';

          if Assigned(MyImg) then
          begin
            MS := TMemoryStream.Create;
            MS.Position:=0;
            MyImg.SaveToStream(MS);
            ret:=StreamToBase64(MS);
            MS.Free;
            MS := nil;
          end;

        finally
              result := ret;
       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

function TSmartDesktopImage.FromStreamToBase64(MyImg: TBitmap): string;
var
   ret : string;
   MS  : TMemoryStream;
begin
     try
        try

          ret := '';

          if Assigned(MyImg) then
          begin
            MS := TMemoryStream.Create;
            MS.Position:=0;
            MyImg.SaveToStream(MS);
            ret:=StreamToBase64(MS);
            MS.Free;
            MS := nil;
          end;

        finally
               result := ret;
       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.FromBase64ToStream(MyImg: TJpegImage; sb64: string
  );
var
    MS  : TMemoryStream;
begin
     try
        try

          if Assigned(MyImg) then
          begin
               MS := TMemoryStream.Create;
               Base64ToStream(sb64,MS);
               MS.Position:=0;
               MyImg.LoadFromStream(MS);
               MS.Free;
               MS := nil;
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TSmartDesktopImage.FromBase64ToStream(MyImg: TBitmap; sb64: string);
var
    MS  : TMemoryStream;
begin
     try
        try

          if Assigned(MyImg) then
          begin
               MS := TMemoryStream.Create;
               Base64ToStream(sb64,MS);
               MS.Position:=0;
               MyImg.LoadFromStream(MS);
               MS.Free;
               MS := nil;
          end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TSmartDesktopImage->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;


end.

