{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uSmartEncrypt;

{$mode objfpc}{$H+}
{$define NoUseBlowFish}


interface

uses
  Classes, SysUtils
  {$ifdef UseBlowFish}
  , BlowFish
  {$else}
  , uaes128
  {$endif}
  ;

{

The blowfish algorithm used here generates blocks in multiples of 8, based on how long the original text is

}

function Crypt(password : string; valore: string)     : string;
function DeCrypt(password : string; valore: string)   : string;
Function MemoryStreamToStringDef(MS: TMemoryStream; const ADefault: string = ''): string;
Function TryMemoryStreamFromString(var MS: TMemoryStream; var s: string): Boolean;

implementation

function Crypt(password: string; valore: string): string;
var
{$ifdef UseBlowFish}
       Inp, Outp, EC: TStream;
{$else}
       app: TLazAES;
{$endif}
begin
  if password='' then
  begin
       result := valore;
  end else begin

       {$ifdef UseBlowFish}

       try
          try
             Inp := TStringStream.Create(valore);
             Outp := TStringStream.Create('');
             EC := TBlowFishEncryptStream.Create(password, Outp);
             Inp.Position := 0;
             EC.CopyFrom(Inp, Inp.Size);

             if Assigned(EC) then
             begin
                  EC.Free;
                  EC := nil;
             end;

             result := TStringStream(Outp).DataString;

          finally

            if Assigned(EC) then
            begin
                 EC.Free;
                 EC := nil;
            end;

            if Assigned(Outp) then
            begin
                 Outp.Free;
                 Outp := nil;
            end;

            if Assigned(Inp) then
            begin
                 Inp.Free;
                 Inp := nil;
            end;

         end;
       except
             on E: Exception do
             begin
                  result := '';
             end;
       end;
       {$else}
       app:=TLazAES.Create(password);
       result:=app.EncriptAES128CTR(valore);
       app.Free;
       {$endif}

  end;
end;

function DeCrypt(password: string; valore: string): string;
var
{$ifdef UseBlowFish}
   Inp, Outp, EC: TStream;
{$else}
   app: TLazAES;
{$endif}
begin
  if password='' then
  begin
       result := valore;
  end else begin
       {$ifdef UseBlowFish}
       try
          try

             Inp := TStringStream.Create(valore);
             Outp := TStringStream.Create('');
             EC := TBlowFishDeCryptStream.Create(password, Inp);
             Inp.Position := 0;
             Outp.CopyFrom(EC, Inp.Size);
             result  := TStringStream(Outp).DataString;

          finally

            if Assigned(EC) then
            begin
                 EC.Free;
                 EC := nil;
            end;

            if Assigned(Outp) then
            begin
                 Outp.Free;
                 Outp := nil;
            end;

            if Assigned(Inp) then
            begin
                 Inp.Free;
                 Inp := nil;
            end;

         end;
       except
             on E: Exception do
             begin
                  result := '';
             end;
       end;
       {$else}
       app:=TLazAES.Create(password);
       result:=app.DecriptAES128CTR(valore);
       app.Free;
       {$endif}
  end;
end;

function MemoryStreamToStringDef(MS: TMemoryStream; const ADefault: string
  ): string;
var
   app : TStringStream;
begin
    if Assigned(MS) then
        try
           app := TStringStream.Create();
           MS.Position:=0;
           MS.SaveToStream(app);
           app.Position:=0;
           result := app.DataString;
           app.Free;
           app:=nil;
        except
              Result := ADefault;
        end
    else Result := ADefault;
end;

function TryMemoryStreamFromString(var MS: TMemoryStream; var s: string): Boolean;
var
   app : TStringStream;
begin
    if Assigned(MS) then
        try
           app := TStringStream.Create(s);
           app.Position:=0;
           app.SaveToStream(MS);
           MS.Position:=0;
           app.Free;
           app:=nil;
           result := true;
        except
              Result := false;
        end
    else Result := false;
end;

end.

