{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uUDPClient;

{$mode objfpc}{$H+}

{$define }

interface

uses
  Classes, SysUtils, UnitUDPTransmission, uSmartDesktopImage, Graphics;

type
    TRemoteIpPort = object
         ip     : string;
         port   : integer;
    end;

type

  { TUDPClient }

  TUDPClient = class(TThread)
  private
    Flocalport                     : string;
    Fremote_ip_port                : array of TRemoteIpPort;
    Fpassword                      : string;
    FappUDP                        : TNGITUDPComponent;
    FAskNotifyError                : TAskNotifyError;
    FSendDesktopLoop               : boolean;
    Fnum_row_cells_screen          : integer;
    Fnum_col_cells_screen          : integer;
    Fidx_zone                      : integer;
    FWith_Mouse_Position           : boolean;
    FMouseDimension                : integer;
    FQuality                       : integer;
    Fmax_char_packet               : integer;

    procedure NotifyError(Sender: TObject; id_error : integer; desc_error : string);
  public
    constructor Create(localport_p: string; password_p: string; SendDesktopLoop_p : boolean; With_Mouse_Position_P : boolean; MouseDimension_P : integer; num_row_cells_screen : integer; num_col_cells_screen : integer; idx_zone : integer; max_char_packet : integer; ImageQuality: integer);
    Destructor Destroy; override;
    procedure Execute; override;
    procedure SetSendDesktopLoop(value : boolean);
    function GetSendDesktopLoop: boolean;
    procedure AddRemoteIpPort(remote_ip : string; remote_port : integer);
  published
    property OnNotifyError: TAskNotifyError read FAskNotifyError write FAskNotifyError;
  end;

implementation

uses
    uMain, usplit, Controls;

{ TUDPClient }

procedure TUDPClient.NotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin
          try
             try

                if Assigned(OnNotifyError) then
                   OnNotifyError(Sender, id_error, desc_error);

             finally

            end;
          except
                on E: Exception do
                begin

                     {$ifdef WriteDebug}
                     writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                     {$endif}
                end;
          end;
end;

constructor TUDPClient.Create(localport_p: string; password_p: string;
  SendDesktopLoop_p: boolean; With_Mouse_Position_P: boolean;
  MouseDimension_P: integer; num_row_cells_screen: integer;
  num_col_cells_screen: integer; idx_zone: integer; max_char_packet: integer;
  ImageQuality: integer);
begin
     try
        try

           inherited Create(true);
           Flocalport                         := localport_p;
           Fpassword                          := password_p;
           FSendDesktopLoop                   := SendDesktopLoop_p;
           Fnum_row_cells_screen              := num_row_cells_screen;
           Fnum_col_cells_screen              := num_col_cells_screen;
           Fidx_zone                          := idx_zone;
           FWith_Mouse_Position               := With_Mouse_Position_P;
           FMouseDimension                    := MouseDimension_P;
           FQuality                           := ImageQuality;
           Fmax_char_packet                   := max_char_packet;

           SetLength(Fremote_ip_port, 0);

           FappUDP                            := TNGITUDPComponent.Create(Flocalport, Fpassword);
           FappUDP.OnNotifyError              := @NotifyError;
        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

destructor TUDPClient.Destroy;
begin
     try
        try

           FappUDP.Free;
           inherited Destroy;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPClient.Execute;
var
  Msg            : string;
  sImg           : TSmartDesktopImage;
  JpgS           : TJpegImage;
  JpgP           : TJpegImage;
  r_ip           : TStringList;
  i              : integer;
  ScreenW        : integer;
  ScreenH        : integer;
begin
     try
        try

           if FappUDP.IsStartedCorrect then
           begin
             while (Self.Terminated=false) do
             begin

                  sImg           := TSmartDesktopImage.Create;
                  JpgS           := TJpegImage.Create;
                  JpgP           := TJpegImage.Create;

                  {$ifdef GetScreenInMainThread}
                          EnterCriticalsection(FrmMain.UDPCriticalSection);

                          FrmMain.ThreadZone             := Fidx_zone;
                          FrmMain.ByPassTimer            := true;
                          Synchronize(@FrmMain.UDPClientGetScreen);
                          Msg                            := FrmMain.ZoneB64;
                          FrmMain.ByPassTimer            := false;
                          ScreenW                        := FrmMain.MyScreenW;
                          ScreenH                        := FrmMain.MyScreenH;

                          LeaveCriticalsection(FrmMain.UDPCriticalSection);

                          sImg.FromBase64ToStream(JpgP, Msg);
                  {$else}
                         sImg.SetQuality(FQuality);
                         sImg.LoadScreen(JpgS);

                         if Self.FWith_Mouse_Position then
                         begin
                              JpgS.Canvas.Brush.Color:= clRed;
                              JpgS.Canvas.Ellipse(Mouse.CursorPos.X, Mouse.CursorPos.Y, Mouse.CursorPos.X+FMouseDimension, Mouse.CursorPos.Y+FMouseDimension);
                         end;

                         sImg.CopyZoneOfScreenInOtherImage(JpgS,Fnum_row_cells_screen,Fnum_col_cells_screen,Fidx_zone + 1,JpgP);

                         ScreenW            := JpgS.Width;
                         ScreenH            := JpgS.Height;
                  {$endif}

                  Msg := '[' + IntToStr(ScreenW) + ';' + IntToStr(ScreenH) + ']';

                  for i:=0 to Length(Fremote_ip_port)-1 do
                  begin
                       FappUDP.SendString(Msg, Fremote_ip_port[i].ip, Fremote_ip_port[i].port);
                       Sleep(4);
                       FappUDP.SendJpg(JpgP, Self.Fmax_char_packet, Fremote_ip_port[i].ip, Fremote_ip_port[i].port);
                  end;

                  JpgP.Free;
                  JpgP := nil;
                  JpgS.Free;
                  JpgS := nil;
                  sImg.Free;
                  sImg := nil;
                  //Sleep(10);

             end;
           end else begin
           end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPClient.SetSendDesktopLoop(value: boolean);
begin
     try
        try

           Self.FSendDesktopLoop:=true;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;

end;

function TUDPClient.GetSendDesktopLoop: boolean;
begin
     try
        try

           result := Self.FSendDesktopLoop;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPClient.AddRemoteIpPort(remote_ip: string; remote_port: integer);
begin
     SetLength(Fremote_ip_port, Length(Fremote_ip_port)+1);
     Fremote_ip_port[Length(Fremote_ip_port)-1].ip   := remote_ip;
     Fremote_ip_port[Length(Fremote_ip_port)-1].port := remote_port;
end;

end.

