{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uUDPComunicationClient;

{$mode objfpc}{$H+}

{$define }

interface

uses
  Classes, SysUtils, UnitUDPTransmission;

type

  { TUDPComunicationClient }

  TUDPComunicationClient = class(TThread)
  private
    Flocalport                     : string;
    Fremoteip                      : string;
    Fremoteport                    : string;
    Fpassword                      : string;
    FappUDP                        : TNGITUDPComponent;
    FAskNotifyError                : TAskNotifyError;
    FUdpHolePunching_ok            : boolean;
    procedure NotifyError(Sender: TObject; id_error : integer; desc_error : string);
  public
    constructor Create(localport_p: string; remoteip_p: string;
  remoteport_p: string; password_p: string);
    Destructor Destroy; override;
    procedure Execute; override;
  published
    property OnNotifyError: TAskNotifyError read FAskNotifyError write FAskNotifyError;
  end;

implementation

uses
    uMain, usplit, Controls;

{ TUDPComunicationClient }

procedure TUDPComunicationClient.NotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin
          try
             try

                if Assigned(OnNotifyError) then
                   OnNotifyError(Sender, id_error, desc_error);

             finally

            end;
          except
                on E: Exception do
                begin

                     {$ifdef WriteDebug}
                     writeln('Error in TUDPComunicationClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                     {$endif}
                end;
          end;
end;

constructor TUDPComunicationClient.Create(localport_p: string; remoteip_p: string;
  remoteport_p: string; password_p: string);
begin
     try
        try

           inherited Create(true);
           Flocalport                         := localport_p;
           Fremoteip                          := remoteip_p;
           Fremoteport                        := remoteport_p;
           Fpassword                          := password_p;


           FappUDP                            := TNGITUDPComponent.Create(Flocalport, Fpassword);
           FappUDP.OnNotifyError              := @NotifyError;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

destructor TUDPComunicationClient.Destroy;
begin
     try
        try

           FappUDP.Free;
           inherited Destroy;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationClient.Execute;
var
  Msg            : string;
  Msg2           : string;
  num_loop       : integer;
  exit_loop      : boolean;
begin
     try
        try
           FUdpHolePunching_ok := false;

           if FappUDP.IsStartedCorrect then
           begin

             while (Self.Terminated=false) do
             begin

                  if FUdpHolePunching_ok then
                  begin
                    EnterCriticalsection(FrmMain.UDPCriticalSectionMouse);
                    //LC=Left Click, RC=Right Click, DC=Double click, MV=only move, KU=key up
                    Msg  := '[' + FrmMain.MouseMsg_Button + ';' + IntToStr(FrmMain.MouseMsg_Width) + ';' + IntToStr(FrmMain.MouseMsg_Height) + ';' + IntToStr(FrmMain.MouseMsg_X) + ';' + IntToStr(FrmMain.MouseMsg_Y) + ';' + FrmMain.LastShiftPressed + ']';

                    if trim(FrmMain.LastKeyPressd)<>'' then
                       Msg2 := '[KU;' + FrmMain.LastKeyPressd + ';' + FrmMain.LastShiftPressed + ';0;0;0]'
                    else
                        Msg2 := '';

                    FrmMain.MouseMsg_Button  := '';
                    FrmMain.LastKeyPressd    := '';
                    FrmMain.LastShiftPressed := '';
                    LeaveCriticalsection(FrmMain.UDPCriticalSectionMouse);

                    //send mouse info
                    num_loop  := 0;
                    exit_loop := false;
                    while (not exit_loop) and (num_loop<=100) do
                    begin
                         exit_loop := FappUDP.SendStringWithResult(Msg, Self.Fremoteip, StrToIntDef(Self.Fremoteport,-1));
                         Sleep(1);
                         inc(num_loop);
                    end;

                    Sleep(4);

                    if Msg2<>'' then
                    begin
                      //send key info
                      num_loop  := 0;
                      exit_loop := false;
                      while (not exit_loop) and (num_loop<=100) do
                      begin
                           exit_loop := FappUDP.SendStringWithResult(Msg2, Self.Fremoteip, StrToIntDef(Self.Fremoteport,-1));
                           Sleep(1);
                           inc(num_loop);
                      end;

                      Sleep(4);
                    end;

                  end;

                  Msg := '[P 0;0;0;0;UDP_HOLE_PUNCHING]';
                  if FappUDP.SendStringWithResult(Msg, Self.Fremoteip, StrToIntDef(Self.Fremoteport,-1)) then
                  begin
                       FUdpHolePunching_ok := true;
                  end;

             end;
           end else begin
           end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationClient->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;


end.

