{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uUDPComunicationServer;

{$mode objfpc}{$H+}

{$define }

interface

uses
  Classes, SysUtils, UnitUDPTransmission, strutils;

type

  { TUDPComunicationServer }

  TUDPComunicationServer = class(TThread)
  private
    Flocalport                    : string;
    Fpassword                     : string;
    FappUDP                       : TNGITUDPComponent;
    FAskNotifyError               : TAskNotifyError;
    FTimeOut                      : integer;
    FLastMsg                      : string;
    FUdpHolePunching_ok           : boolean;
    FValidRemoteIp                : string;
    FValidRemotePort              : integer;
    procedure ProcessMessagesMouse(TypeMsg: string; Width, Height, X, Y: integer; Shift: string);
    procedure ProcessMessagesKey(TypeMsg: string; Key: string; Shift: string);
    procedure NotifyError(Sender: TObject; id_error : integer; desc_error : string);
    function MyFastCopy(value: string; from_pos: integer; to_pos: integer;
      var my_value: string; dont_copy_first_char_returned: boolean = false): string;
  public
    constructor Create(localport_p: string; password_p: string);
    Destructor Destroy; override;
    procedure Execute; override;
    procedure SetTimeOut(TimeOut : integer);
    procedure SetValidConnection(ValidRemoteIp : string; ValidRemotePort : integer);
  published
    property OnNotifyError: TAskNotifyError read FAskNotifyError write FAskNotifyError;
  end;

implementation

uses
    uMain;

{ TUDPComunicationServer }
constructor TUDPComunicationServer.Create(localport_p: string; password_p: string);
begin
     try
        try

           inherited Create(true); //not automatic start
           Self.SetTimeOut(1000);
           Flocalport                    := localport_p;
           Fpassword                     := password_p;
           FappUDP                       := TNGITUDPComponent.Create(Flocalport, Fpassword);
           FappUDP.OnNotifyError         :=@NotifyError;
           FValidRemoteIp                := '';
           FValidRemotePort              := 0;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

destructor TUDPComunicationServer.Destroy;
begin
     try
        try

           FappUDP.Free;
           inherited Destroy;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.Execute;
var
   Msgs       : string;
   Msg        : string;
   R_IP       : string;
   R_Port     : integer;
   Width      : integer;
   Height     : integer;
   X          : integer;
   Y          : integer;
   TypeMsg    : string;
   position   : integer;
   app        : string;
   key        : string;
   Shift      : string;
begin
     try
        try
           FUdpHolePunching_ok := false;

           if FappUDP.IsStartedCorrect then
           begin

             while (Self.Terminated=false) do
             begin

                  //Msgs := FappUDP.GetString(R_IP, R_Port, Msgs);
                  if not FappUDP.GetStringWithResult(Self.FValidRemoteIp, Self.FValidRemotePort, Msgs) then
                  begin
                       if (not FUdpHolePunching_ok) then
                       begin
                         Msg := '[P 0;0;0;0;UDP_HOLE_PUNCHING]';
                         if FappUDP.SendStringWithResult(Msg,Self.FValidRemoteIp, Self.FValidRemotePort) then
                            FUdpHolePunching_ok := true;
                       end;
                  end;


                  while (Msgs<>'') do
                  begin
                       if (pos('[', Msgs)>0) and (pos(']', Msgs)>0) then
                       begin
                         position    := Pos(']',Msgs);
                         Msgs        := MyFastCopy(Msgs,pos('[', Msgs)+1, position-1, Msg);
                         position    := Pos(']',Msgs);
                         Msgs        := Copy(Msgs, position+1);

                         if Length(Msg)>4 then
                         begin
                              if WordCount(Msg,[';'])>=5 then
                              begin
                                Msg         := MyFastCopy(Msg, 1, 2, TypeMsg, true);

                                //MD = mouse down, LC=Left Click, RC=Right Click, DC=Double click, MV=only move , KU=key up
                                if (TypeMsg='MV')or(TypeMsg='MD')or(TypeMsg='LC')or(TypeMsg='RC')or(TypeMsg='DC') then
                                begin
                                  position    := Pos(';',Msg);
                                  Msg         := MyFastCopy(Msg, 1, position-1 , app, true);
                                  Width       := StrToIntDef(app,0);

                                  position    := Pos(';',Msg);
                                  Msg         := MyFastCopy(Msg, 1, position-1 , app, true);
                                  Height      := StrToIntDef(app,0);

                                  position    := Pos(';',Msg);
                                  Msg         := MyFastCopy(Msg, 1, position-1 , app, true);
                                  X           := StrToIntDef(app,-1);

                                  position    := Pos(';',Msg);
                                  Shift       := MyFastCopy(Msg, 1, position-1 , app, true);
                                  Y           := StrToIntDef(app,-1);
                                end;

                                if (TypeMsg='KU') then
                                begin

                                  position    := Pos(';',Msg);
                                  Msg         := MyFastCopy(Msg, 1, position-1 , key, true);

                                  position    := Pos(';',Msg);
                                  Msg         := MyFastCopy(Msg, 1, position-1 , Shift, true);

                                end;

                                case TypeMsg of
                                     'MV','MD','LC','RC','DC' : begin
                                          //mouse action
                                          //MD = mouse down, LC=Left Click, RC=Right Click, DC=Double click, MV=only move
                                          if (FValidRemoteIp<>'') and (FValidRemotePort>0) then
                                          begin
                                               if (Width>0) and (Height>0) and (X>=0) and (Y>=0) then
                                               begin
                                                    Self.ProcessMessagesMouse(TypeMsg, Width, Height, X, Y, Shift);
                                               end;
                                          end;
                                     end;

                                     'KU' : begin

                                          if (FValidRemoteIp<>'') and (FValidRemotePort>0) then
                                          begin
                                               if (key<>'') then
                                               begin
                                                    Self.ProcessMessagesKey(TypeMsg, key, Shift);
                                               end;
                                          end;

                                     end;

                                     'P ' : begin
                                           //udp hole punching
                                          Msg := '[P 0;0;0;0;UDP_HOLE_PUNCHING]';
                                          if FappUDP.SendStringWithResult(Msg,Self.FValidRemoteIp, Self.FValidRemotePort) then
                                             FUdpHolePunching_ok := true;
                                     end;

                                end;


                              end;

                         end;

                       end else begin
                            Msgs := '';
                       end;

                  end;


             end;
           end else begin
           end;

        finally

       end;
     except
           on E: Exception do
           begin
                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.ProcessMessagesMouse(TypeMsg: string; Width,
  Height, X, Y: integer; Shift: string);
begin
     try
        try

           EnterCriticalsection(FrmMain.UDPCriticalSectionMouse);

           FrmMain.MouseMsg_Button  := TypeMsg;
           FrmMain.MouseMsg_Width   := Width;
           FrmMain.MouseMsg_Height  := Height;
           FrmMain.MouseMsg_X       := X;
           FrmMain.MouseMsg_Y       := Y;
           FrmMain.LastShiftPressed := Shift;

           Synchronize(@FrmMain.UDPActionMouse);
           LeaveCriticalsection(FrmMain.UDPCriticalSectionMouse);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.ProcessMessagesKey(TypeMsg: string; Key: string;
  Shift: string);
begin
     try
        try

           EnterCriticalsection(FrmMain.UDPCriticalSectionMouse);

           FrmMain.LastShiftPressed := Shift;
           FrmMain.LastKeyPressd    := key;

           Synchronize(@FrmMain.UDPActionKey);
           LeaveCriticalsection(FrmMain.UDPCriticalSectionMouse);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.NotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin
     try
        try

           if Assigned(OnNotifyError) then
              OnNotifyError(Sender, id_error, desc_error);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;

end;

function TUDPComunicationServer.MyFastCopy(value: string; from_pos: integer;
  to_pos: integer; var my_value: string; dont_copy_first_char_returned: boolean): string;
var
   len : integer;
   i   : integer;
begin
     try
        try

           result := '';
           my_value := '';
           len    := Length(value);
           i      := 1;
           while (i<=len) do
           begin
                if i>=from_pos then
                begin
                     if (i<=to_pos) then
                     begin
                          my_value := my_value + value[i];
                     end else begin
                          if (i=to_pos+1)and(dont_copy_first_char_returned) then
                          begin

                          end else begin
                              result := result + value[i];
                          end;
                     end;
                end;

                Inc(i);
           end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.SetTimeOut(TimeOut: integer);
begin
     try
        try

           Self.FTimeOut:=TimeOut;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPComunicationServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPComunicationServer.SetValidConnection(ValidRemoteIp: string;
  ValidRemotePort: integer);
begin
     Self.FValidRemoteIp    := ValidRemoteIp;
     Self.FValidRemotePort  := ValidRemotePort;
end;

end.

