{

 Copyright 2020 Sammarco Francesco (francesco.sammarco@gmail.com)

 This file is part of Korg Remote Desktop.

 Korg Remote Desktop is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Korg Remote Desktop is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Korg Remote Desktop.  If not, see <http://www.gnu.org/licenses/>.

}

unit uUDPServer;

{$mode objfpc}{$H+}

{$define }

interface

uses
  Classes, SysUtils, UnitUDPTransmission, strutils, uSmartDesktopImage, Graphics;

type

  { TUDPServer }

  TUDPServer = class(TThread)
  private
    Flocalport                    : string;
    Fpassword                     : string;
    FappUDP                       : TNGITUDPComponent;
    FAskNotifyError               : TAskNotifyError;
    FTimeOut                      : integer;
    Fnum_row_cells_screen         : integer;
    Fnum_col_cells_screen         : integer;
    Fidx_zone                     : integer;
    FRemoteScreenWidth            : integer;
    FRemoteScreenHeight           : integer;
    Fremote_ip                    : string;
    Fremote_port                  : string;
    procedure NotifyError(Sender: TObject; id_error : integer; desc_error : string);
    function MyFastCopy(value: string; from_pos: integer; to_pos: integer;
      var my_value: string; dont_copy_first_char_returned: boolean = false): string;
  public
    constructor Create(localport_p: string; remote_ip_p : string; remote_port_p : string ;password_p: string; num_row_cells_screen : integer; num_col_cells_screen : integer; idx_zone : integer{; max_char_packet : integer});
    Destructor Destroy; override;
    procedure Execute; override;
    procedure SetTimeOut(TimeOut : integer);
  published
    property OnNotifyError: TAskNotifyError read FAskNotifyError write FAskNotifyError;
  end;

implementation

uses
    uMain;

{ TUDPServer }
constructor TUDPServer.Create(localport_p: string;
  remote_ip_p: string; remote_port_p: string; password_p: string; num_row_cells_screen: integer;
  num_col_cells_screen: integer; idx_zone: integer{; max_char_packet: integer});
begin
     try
        try

           inherited Create(true); //not automatic start
           Self.SetTimeOut(1000);
           Flocalport                    := localport_p;
           Fpassword                     := password_p;
           FappUDP                       := TNGITUDPComponent.Create(Flocalport, Fpassword);
           FappUDP.OnNotifyError         :=@NotifyError;
           Fnum_row_cells_screen         := num_row_cells_screen;
           Fnum_col_cells_screen         := num_col_cells_screen;
           Fidx_zone                     := idx_zone;
           Fremote_ip                    := remote_ip_p;
           Fremote_port                  := remote_port_p;


        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

destructor TUDPServer.Destroy;
begin
     try
        try

           FappUDP.Free;
           inherited Destroy;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPServer.Execute;
var
   Msgs          : string;
   Msg           : string;
   R_IP          : string;
   R_Port        : integer;
   position      : integer;
   app           : string;
   JpgS          : TJpegImage;
   sImg          : TSmartDesktopImage;
   continue      : boolean;
begin
     try
        try

           if FappUDP.IsStartedCorrect then
           begin

             while (Self.Terminated=false) do
             begin

                  if (FRemoteScreenWidth>0) and (FRemoteScreenHeight>0) then
                  begin

                    JpgS           := TJpegImage.Create;
                    if (FappUDP.GetJpg(R_IP, R_PORT, JpgS, Self.FTimeOut)) then
                    begin

                      continue := true;
                      {$ifdef  FilterFromIpPort}
                           if (R_IP <> Self.Fremote_ip) or (R_Port <> StrToIntDef(Self.Fremote_port,0)) then
                           begin
                                continue := false;
                           end;
                      {$endif}

                      if continue then
                      begin
                        sImg           := TSmartDesktopImage.Create;

                        EnterCriticalsection(FrmMain.UDPCriticalSection);
                        if not FrmMain.StopUploadImage then
                        begin
                          {$ifdef Windows}
                          //FrmMain.UDPServerMessage   := full_message;
                          Jpgs.SaveToStream(FrmMain.MemoryJpg);
                          FrmMain.MemoryJpg.Position := 0;
                          FrmMain.UDPServerIdx       := Self.Fidx_zone;
                          FrmMain.RemoteScreenWidth  := FRemoteScreenWidth;
                          FrmMain.RemoteScreenHeight := FRemoteScreenHeight;
                          Synchronize(@FrmMain.UDPServerShowImage);
                          {$else}
                                 {$ifdef GetScreenInMainThread}
                                 //FrmMain.UDPServerMessage   := full_message;
                                 Jpgs.SaveToStream(FrmMain.MemoryJpg);
                                 FrmMain.MemoryJpg.Position := 0;
                                 FrmMain.UDPServerIdx       := Self.Fidx_zone;
                                 FrmMain.RemoteScreenWidth  := FRemoteScreenWidth;
                                 FrmMain.RemoteScreenHeight := FRemoteScreenHeight;
                                 Synchronize(@FrmMain.UDPServerShowImage);
                                 {$else}
                                        sImg.RefreshZoneOfScreenWithImage(JpgS, COUNT_ROW_ZONE_SCREEN, COUNT_COL_ZONE_SCREEN,(Self.Fidx_zone+1),FrmMain.Image1.Picture.Jpeg, FRemoteScreenWidth, FRemoteScreenHeight);  //le ultime due sono le dimensioni del monitor remoto
                                        FrmMain.Image1.Invalidate;
                                        Synchronize(@FrmMain.AdjustScreenImage);
                                 {$endif}
                          {$endif}

                        end;
                        LeaveCriticalsection(FrmMain.UDPCriticalSection);

                        sImg.Free;
                        sImg:=nil;
                      end;


                    end;
                    JpgS.Free;
                    JpgS := nil;
                  end else begin
                     Msgs := FappUDP.GetString(R_IP, R_Port, Self.FTimeOut);
                     {$ifdef FilterFromIpPort}
                          if (R_IP <> Self.Fremote_ip) or (R_Port <> StrToIntDef(Self.Fremote_port,0)) then
                          begin
                               Msgs := '';
                          end;
                     {$endif}

                     while (Msgs<>'') do
                     begin

                          if (pos('[', Msgs)>0) and (pos(']', Msgs)>0) then
                          begin
                            position    := Pos(']',Msgs);
                            Msgs        := MyFastCopy(Msgs,pos('[', Msgs)+1, position-1, Msg);
                            position    := Pos(']',Msgs);
                            Msgs        := Copy(Msgs, position+1);

                            if Length(Msg)>4 then
                            begin

                                 if WordCount(Msg,[';'])=2 then
                                 begin

                                   position                := Pos(';',Msg);
                                   Msg                     := MyFastCopy(Msg, 1, position-1 , app, true);
                                   FRemoteScreenWidth      := StrToIntDef(app,-1);
                                   FRemoteScreenHeight     := StrToIntDef(Msg,-1);

                                 end;

                            end;

                          end else begin
                               Msgs := '';
                          end;

                     end;

                     Msg := '[P ;0;0;0;UDP_HOLE_PUNCHING]';
                     FappUDP.SendString(Msg,Self.Fremote_ip, StrToIntDef(Self.Fremote_port,0));

                  end;

             end;
           end else begin
           end;

        finally

       end;
     except
           on E: Exception do
           begin
                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPServer.NotifyError(Sender: TObject; id_error: integer;
  desc_error: string);
begin
     try
        try

           if Assigned(OnNotifyError) then
              OnNotifyError(Sender, id_error, desc_error);

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;

end;

function TUDPServer.MyFastCopy(value: string; from_pos: integer;
  to_pos: integer; var my_value: string; dont_copy_first_char_returned: boolean): string;
var
   len : integer;
   i   : integer;
begin
     try
        try

           result := '';
           my_value := '';
           len    := Length(value);
           i      := 1;
           while (i<=len) do
           begin
                if i>=from_pos then
                begin
                     if (i<=to_pos) then
                     begin
                          my_value := my_value + value[i];
                     end else begin
                          if (i=to_pos+1)and(dont_copy_first_char_returned) then
                          begin

                          end else begin
                              result := result + value[i];
                          end;
                     end;
                end;

                Inc(i);
           end;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

procedure TUDPServer.SetTimeOut(TimeOut: integer);
begin
     try
        try

           Self.FTimeOut:=TimeOut;

        finally

       end;
     except
           on E: Exception do
           begin

                {$ifdef WriteDebug}
                writeln('Error in TUDPServer->' , {$I %CURRENTROUTINE%}, ': ' + E.Message);
                {$endif}
           end;
     end;
end;

end.

